@@ -1,5 +1,5 @@
 ;;; GNU Guix --- Functional package management for GNU
-;;; Copyright © 2014 Ludovic Courtès <ludo@gnu.org>
+;;; Copyright © 2014, 2015 Ludovic Courtès <ludo@gnu.org>
 ;;;
 ;;; This file is part of GNU Guix.
 ;;;
@@ -23,13 +23,13 @@
 ;;;
 
 (use-modules (gnu))
-(use-service-modules desktop xorg networking avahi)
+(use-service-modules desktop xorg networking avahi dbus)
 (use-package-modules linux xorg tor avahi)
 
 (operating-system
  (host-name "gnu")
  (timezone "Europe/Paris")
- (locale "en_US.UTF-8")
+ (locale "en_US.utf8")
 
  (bootloader (grub-configuration
               (device "/dev/sda")))
@@ -71,7 +71,7 @@ You can log in as 'guest' or 'root' with no password.
                                              #:gateway "10.0.2.2")
 
                   (avahi-service)
-                  (dbus-service (list avahi))
+                  (dbus-service)
                   (tor-service)
 
                   %base-services))
