@@ -3,7 +3,7 @@
 ;;; Code:
 
 
-;;;### (autoloads nil "guix" "guix.el" (21610 7897 899234 366000))
+;;;### (autoloads nil "guix" "guix.el" (22073 56029 825740 602000))
 ;;; Generated autoloads from guix.el
 
 (autoload 'guix-search-by-name "guix" "\
@@ -76,12 +76,27 @@ Interactively with prefix, prompt for PROFILE.
 
 \(fn FROM TO &optional PROFILE)" t nil)
 
+(autoload 'guix-edit "guix" "\
+Edit (go to location of) package with ID-OR-NAME.
+
+\(fn ID-OR-NAME)" t nil)
+
 ;;;***
 
-;;;### (autoloads nil "guix-base" "guix-base.el" (21705 20255 266858
-;;;;;;  174000))
+;;;### (autoloads nil "guix-base" "guix-base.el" (22073 56029 825740
+;;;;;;  602000))
 ;;; Generated autoloads from guix-base.el
 
+(autoload 'guix-apply-manifest "guix-base" "\
+Apply manifest from FILE to PROFILE.
+This function has the same meaning as 'guix package --manifest' command.
+See Info node `(guix) Invoking guix package' for details.
+
+Interactively, use the current profile and prompt for manifest
+FILE.  With a prefix argument, also prompt for PROFILE.
+
+\(fn PROFILE FILE &optional OPERATION-BUFFER)" t nil)
+
 (autoload 'guix-pull "guix-base" "\
 Run Guix pull operation.
 If VERBOSE is non-nil (with prefix argument), produce verbose output.
@@ -90,12 +105,84 @@ If VERBOSE is non-nil (with prefix argument), produce verbose output.
 
 ;;;***
 
-;;;### (autoloads nil "guix-emacs" "guix-emacs.el" (21705 20255 266858
-;;;;;;  174000))
+;;;### (autoloads nil "guix-build-log" "guix-build-log.el" (22073
+;;;;;;  56029 825740 602000))
+;;; Generated autoloads from guix-build-log.el
+
+(autoload 'guix-build-log-mode "guix-build-log" "\
+Major mode for viewing Guix build logs.
+
+\\{guix-build-log-mode-map}
+
+\(fn)" t nil)
+
+(autoload 'guix-build-log-minor-mode "guix-build-log" "\
+Toggle Guix Build Log minor mode.
+
+With a prefix argument ARG, enable Guix Build Log minor mode if
+ARG is positive, and disable it otherwise.  If called from Lisp,
+enable the mode if ARG is omitted or nil.
+
+When Guix Build Log minor mode is enabled, it highlights build
+log in the current buffer.  This mode can be enabled
+programmatically using hooks:
+
+  (add-hook 'shell-mode-hook 'guix-build-log-minor-mode)
+
+\\{guix-build-log-minor-mode-map}
+
+\(fn &optional ARG)" t nil)
+
+(autoload 'guix-build-log-minor-mode-activate-maybe "guix-build-log" "\
+Activate `guix-build-log-minor-mode' depending on
+`guix-build-log-minor-mode-activate' variable.
+
+\(fn)" nil nil)
+
+(add-to-list 'auto-mode-alist (cons (rx "/guix/drvs/" (= 2 alnum) "/" (= 30 alnum) "-" (+ (any alnum "-+.")) ".drv" string-end) 'guix-build-log-mode))
+
+;;;***
+
+;;;### (autoloads nil "guix-command" "guix-command.el" (22073 56029
+;;;;;;  825740 602000))
+;;; Generated autoloads from guix-command.el
+ (autoload 'guix "guix-command" "Popup window for 'guix'." t)
+
+;;;***
+
+;;;### (autoloads nil "guix-devel" "guix-devel.el" (22073 56029 825740
+;;;;;;  602000))
+;;; Generated autoloads from guix-devel.el
+
+(autoload 'guix-devel-mode "guix-devel" "\
+Minor mode for `scheme-mode' buffers.
+
+With a prefix argument ARG, enable the mode if ARG is positive,
+and disable it otherwise.  If called from Lisp, enable the mode
+if ARG is omitted or nil.
+
+When Guix Devel mode is enabled, it provides the following key
+bindings:
+
+\\{guix-devel-mode-map}
+
+\(fn &optional ARG)" t nil)
+
+(autoload 'guix-devel-activate-mode-maybe "guix-devel" "\
+Activate `guix-devel-mode' depending on
+`guix-devel-activate-mode' variable.
+
+\(fn)" nil nil)
+
+;;;***
+
+;;;### (autoloads nil "guix-emacs" "guix-emacs.el" (21934 40677 836015
+;;;;;;  392000))
 ;;; Generated autoloads from guix-emacs.el
 
 (autoload 'guix-emacs-load-autoloads "guix-emacs" "\
 Load autoloads for Emacs packages installed in a user profile.
+Add autoloads directories to `load-path'.
 If ALL is nil, activate only those packages that were installed
 after the last activation, otherwise activate all Emacs packages
 installed in `guix-user-profile'.
@@ -104,8 +191,19 @@ installed in `guix-user-profile'.
 
 ;;;***
 
-;;;### (autoloads nil "guix-prettify" "guix-prettify.el" (21610 7897
-;;;;;;  899234 366000))
+;;;### (autoloads nil "guix-pcomplete" "guix-pcomplete.el" (22073
+;;;;;;  56029 825740 602000))
+;;; Generated autoloads from guix-pcomplete.el
+
+(autoload 'pcomplete/guix "guix-pcomplete" "\
+Completion for `guix'.
+
+\(fn)" nil nil)
+
+;;;***
+
+;;;### (autoloads nil "guix-prettify" "guix-prettify.el" (22073 56029
+;;;;;;  825740 602000))
 ;;; Generated autoloads from guix-prettify.el
 
 (autoload 'guix-prettify-mode "guix-prettify" "\
@@ -157,9 +255,11 @@ See `guix-prettify-mode' for more information on Guix-Prettify mode.
 
 ;;;***
 
-;;;### (autoloads nil nil ("guix-backend.el" "guix-history.el" "guix-info.el"
-;;;;;;  "guix-init.el" "guix-list.el" "guix-messages.el" "guix-profiles.el"
-;;;;;;  "guix-utils.el") (21705 20360 6189 205000))
+;;;### (autoloads nil nil ("guix-backend.el" "guix-config.el" "guix-external.el"
+;;;;;;  "guix-geiser.el" "guix-guile.el" "guix-help-vars.el" "guix-history.el"
+;;;;;;  "guix-info.el" "guix-init.el" "guix-list.el" "guix-messages.el"
+;;;;;;  "guix-popup.el" "guix-profiles.el" "guix-read.el" "guix-utils.el")
+;;;;;;  (22073 56147 342847 820000))
 
 ;;;***
 
