@@ -33,7 +33,7 @@
 (define-public boost
   (package
     (name "boost")
-    (version "1.57.0")
+    (version "1.58.0")
     (source (origin
               (method url-fetch)
               (uri (string-append
@@ -42,7 +42,8 @@
                     ".tar.bz2"))
               (sha256
                (base32
-                "0rs94vdmg34bwwj23fllva6mhrml2i7mvmlb11zyrk1k5818q34i"))))
+                "1rfkqxns60171q62cppiyzj8pmsbwp1l8jd7p6crriryqd7j1z7x"))
+              (patches (list (search-patch "boost-mips-avoid-m32.patch")))))
     (build-system gnu-build-system)
     (inputs `(("zlib" ,zlib)))
     (native-inputs
@@ -106,14 +107,14 @@ across a broad spectrum of applications.")
 (define-public mdds
   (package
     (name "mdds")
-    (version "0.12.0")
+    (version "0.12.1")
     (source (origin
              (method url-fetch)
              (uri (string-append
                    "http://kohei.us/files/mdds/src/mdds_" version ".tar.bz2"))
              (sha256
               (base32
-               "10ar7r0gkdl2r7916jlkl5c38cynrh7x9s90a5i8d242r8ixw8ia"))))
+               "0gg8mb9kxh3wggh7njj1gf90xy27p0yq2cw88wqar9hhg2fmwmi3"))))
     (build-system gnu-build-system)
     (propagated-inputs
       `(("boost" ,boost))) ; inclusion of header files
