@@ -3,6 +3,7 @@
 ;;; Copyright © 2014 Mark H Weaver <mhw@netris.org>
 ;;; Copyright © 2014 Eric Bavier <bavier@member.fsf.org>
 ;;; Copyright © 2014 Ian Denhardt <ian@zenhack.net>
+;;; Copyright © 2015 Sou Bunnbu <iyzsong@gmail.com>
 ;;;
 ;;; This file is part of GNU Guix.
 ;;;
@@ -35,15 +36,15 @@
 (define-public cmake
   (package
     (name "cmake")
-    (version "2.8.12")
+    (version "3.3.2")
     (source (origin
              (method url-fetch)
-             (uri (string-append
-                   "http://www.cmake.org/files/v"
-                   (version-major+minor version)
-                   "/cmake-" version ".tar.gz"))
+             (uri (string-append "https://www.cmake.org/files/v"
+                                 (version-major+minor version)
+                                 "/cmake-" version ".tar.gz"))
              (sha256
-              (base32 "11q21vyrr6c6smyjy81k2k07zmn96ggjia9im9cxwvj0n88bm1fq"))
+              (base32
+               "08pwy9ip9cgwgynhn5vrjw8drw29gijy1rmziq22n65zds6ifnp7"))
              (patches (list (search-patch "cmake-fix-tests.patch")))))
     (build-system gnu-build-system)
     (arguments
@@ -62,10 +63,12 @@
                          "Source/cmGlobalXCodeGenerator.cxx"
                          "Source/CTest/cmCTestBatchTestHandler.cxx"
                          "Source/cmLocalUnixMakefileGenerator3.cxx"
+                         "Source/cmExecProgramCommand.cxx"
                          "Utilities/cmbzip2/Makefile-libbz2_so"
                          "Utilities/Release/release_cmake.cmake"
                          "Utilities/cmlibarchive/libarchive/archive_write_set_format_shar.c"
-                         "Tests/CMakeLists.txt")
+                         "Tests/CMakeLists.txt"
+                         "Tests/RunCMake/File_Generate/RunCMakeTest.cmake")
                      (("/bin/sh") (which "sh"))))
                  (alist-cons-before
                   'configure 'set-paths
@@ -73,7 +76,9 @@
                     ;; Help cmake's bootstrap process to find system libraries
                     (begin
                       (setenv "CMAKE_LIBRARY_PATH" (getenv "LIBRARY_PATH"))
-                      (setenv "CMAKE_INCLUDE_PATH" (getenv "CPATH"))))
+                      (setenv "CMAKE_INCLUDE_PATH" (getenv "CPATH"))
+                      ;; Get verbose output from failed tests
+                      (setenv "CTEST_OUTPUT_ON_FAILURE" "TRUE")))
                   (alist-replace
                    'configure
                    (lambda* (#:key outputs #:allow-other-keys)
@@ -82,6 +87,7 @@
                                "./configure"
                                (string-append "--prefix=" out)
                                "--system-libs"
+                               "--no-system-jsoncpp" ; not packaged yet
                                ;; By default, the man pages and other docs land
                                ;; in PREFIX/man and PREFIX/doc, but we want them
                                ;; in share/{man,doc}.  Note that unlike
@@ -100,6 +106,10 @@
        ("expat"      ,expat)
        ("bzip2"      ,bzip2)
        ("libarchive" ,libarchive)))
+    (native-search-paths
+     (list (search-path-specification
+             (variable "CMAKE_PREFIX_PATH")
+             (files '("")))))
     (home-page "http://www.cmake.org/")
     (synopsis "Cross-platform build system")
     (description
