@@ -36,9 +36,13 @@
             cross-libc
             cross-gcc))
 
+(define %xgcc
+  ;; GCC package used as the basis for cross-compilation.  It doesn't have to
+  ;; be 'gcc' and can be a specific variant such as 'gcc-4.8'.
+  gcc)
+
 (define (cross p target)
   (package (inherit p)
-    (location (source-properties->location (current-source-location)))
     (name (string-append (package-name p) "-cross-" target))
     (arguments
      (substitute-keyword-arguments (package-arguments p)
@@ -86,97 +90,109 @@ may be either a libc package or #f.)"
   ;; Set the current target system so that 'glibc-dynamic-linker' returns the
   ;; right name.
   (parameterize ((%current-target-system target))
-    (substitute-keyword-arguments (package-arguments gcc-4.8)
-      ((#:configure-flags flags)
-       `(append (list ,(string-append "--target=" target)
-                      ,@(if libc
-                            '()
-                            `( ;; Disable features not needed at this stage.
-                              "--disable-shared" "--enable-static"
+    ;; Disable stripping as this can break binaries, with object files of
+    ;; libgcc.a showing up as having an unknown architecture.  See
+    ;; <http://lists.fedoraproject.org/pipermail/arm/2010-August/000663.html>
+    ;; for instance.
+    (let ((args `(#:strip-binaries? #f
+                  ,@(package-arguments %xgcc))))
+     (substitute-keyword-arguments args
+       ((#:configure-flags flags)
+        `(append (list ,(string-append "--target=" target)
+                       ,@(if libc
+                             `( ;; Disable libcilkrts because it is not
+                                ;; ported to GNU/Hurd. 
+                               "--disable-libcilkrts")
+                             `( ;; Disable features not needed at this stage.
+                               "--disable-shared" "--enable-static"
 
-                              ;; Disable C++ because libstdc++'s configure
-                              ;; script otherwise fails with "Link tests are not
-                              ;; allowed after GCC_NO_EXECUTABLES."
-                              "--enable-languages=c"
+                               ;; Disable C++ because libstdc++'s configure
+                               ;; script otherwise fails with "Link tests are not
+                               ;; allowed after GCC_NO_EXECUTABLES."
+                               "--enable-languages=c"
 
-                              "--disable-threads"  ;libgcc, would need libc
-                              "--disable-libatomic"
-                              "--disable-libmudflap"
-                              "--disable-libgomp"
-                              "--disable-libssp"
-                              "--disable-libquadmath"
-                              "--disable-decimal-float" ;would need libc
-                              )))
+                               "--disable-threads" ;libgcc, would need libc
+                               "--disable-libatomic"
+                               "--disable-libmudflap"
+                               "--disable-libgomp"
+                               "--disable-libssp"
+                               "--disable-libquadmath"
+                               "--disable-decimal-float" ;would need libc
+                               "--disable-libcilkrts"
+                               )))
 
-                ,(if libc
-                     flags
-                     `(remove (cut string-match "--enable-languages.*" <>)
-                              ,flags))))
-      ((#:make-flags flags)
-       (if libc
-           `(let ((libc (assoc-ref %build-inputs "libc")))
-              ;; FLAGS_FOR_TARGET are needed for the target libraries to receive
-              ;; the -Bxxx for the startfiles.
-              (cons (string-append "FLAGS_FOR_TARGET=-B" libc "/lib")
-                    ,flags))
-           flags))
-      ((#:phases phases)
-       (let ((phases
-              `(alist-cons-after
-                'install 'make-cross-binutils-visible
-                (lambda* (#:key outputs inputs #:allow-other-keys)
-                  (let* ((out      (assoc-ref outputs "out"))
-                         (libexec  (string-append out "/libexec/gcc/"
-                                                  ,target))
-                         (binutils (string-append
-                                    (assoc-ref inputs "binutils-cross")
-                                    "/bin/" ,target "-")))
-                    (for-each (lambda (file)
-                                (symlink (string-append binutils file)
-                                         (string-append libexec "/"
-                                                        file)))
-                              '("as" "ld" "nm"))
-                    #t))
-                ,phases)))
-         (if libc
-             `(alist-cons-before
-               'configure 'set-cross-path
-               (lambda* (#:key inputs #:allow-other-keys)
-                 ;; Add the cross Linux headers to CROSS_CPATH, and remove them
-                 ;; from CPATH.
-                 (let ((libc  (assoc-ref inputs "libc"))
-                       (linux (assoc-ref inputs
-                                         "libc/linux-headers")))
-                   (define (cross? x)
-                     ;; Return #t if X is a cross-libc or cross Linux.
-                     (or (string-prefix? libc x)
-                         (string-prefix? linux x)))
+                 ,(if libc
+                      flags
+                      `(remove (cut string-match "--enable-languages.*" <>)
+                               ,flags))))
+       ((#:make-flags flags)
+        (if libc
+            `(let ((libc (assoc-ref %build-inputs "libc")))
+               ;; FLAGS_FOR_TARGET are needed for the target libraries to receive
+               ;; the -Bxxx for the startfiles.
+               (cons (string-append "FLAGS_FOR_TARGET=-B" libc "/lib")
+                     ,flags))
+            flags))
+       ((#:phases phases)
+        (let ((phases
+               `(alist-cons-after
+                 'install 'make-cross-binutils-visible
+                 (lambda* (#:key outputs inputs #:allow-other-keys)
+                   (let* ((out      (assoc-ref outputs "out"))
+                          (libexec  (string-append out "/libexec/gcc/"
+                                                   ,target))
+                          (binutils (string-append
+                                     (assoc-ref inputs "binutils-cross")
+                                     "/bin/" ,target "-"))
+                          (wrapper  (string-append
+                                     (assoc-ref inputs "ld-wrapper-cross")
+                                     "/bin/" ,target "-ld")))
+                     (for-each (lambda (file)
+                                 (symlink (string-append binutils file)
+                                          (string-append libexec "/"
+                                                         file)))
+                               '("as" "nm"))
+                     (symlink wrapper (string-append libexec "/ld"))
+                     #t))
+                 (alist-replace
+                  'install
+                  (lambda _
+                    ;; Unlike our 'strip' phase, this will do the right thing
+                    ;; for cross-compilers.
+                    (zero? (system* "make" "install-strip")))
+                  ,phases))))
+          (if libc
+              `(alist-cons-before
+                'configure 'set-cross-path
+                (lambda* (#:key inputs #:allow-other-keys)
+                  ;; Add the cross Linux headers to CROSS_CPATH, and remove them
+                  ;; from CPATH.
+                  (let ((libc  (assoc-ref inputs "libc"))
+                        (linux (assoc-ref inputs "xlinux-headers")))
+                    (define (cross? x)
+                      ;; Return #t if X is a cross-libc or cross Linux.
+                      (or (string-prefix? libc x)
+                          (string-prefix? linux x)))
 
-                   (setenv "CROSS_CPATH"
-                           (string-append libc "/include:"
-                                          linux "/include"))
-                   (setenv "CROSS_LIBRARY_PATH"
-                           (string-append libc "/lib"))
+                    (setenv "CROSS_CPATH"
+                            (string-append libc "/include:"
+                                           linux "/include"))
+                    (setenv "CROSS_LIBRARY_PATH"
+                            (string-append libc "/lib"))
 
-                   (let ((cpath   (search-path-as-string->list
-                                   (getenv "CPATH")))
-                         (libpath (search-path-as-string->list
-                                   (getenv "LIBRARY_PATH"))))
-                     (setenv "CPATH"
-                             (list->search-path-as-string
-                              (remove cross? cpath) ":"))
-                     (setenv "LIBRARY_PATH"
-                             (list->search-path-as-string
-                              (remove cross? libpath) ":"))
-                     #t)))
-               ,phases)
-             phases)))
-      ((#:strip-binaries? _)
-       ;; Disable stripping as this can break binaries, with object files of
-       ;; libgcc.a showing up as having an unknown architecture.  See
-       ;; <http://lists.fedoraproject.org/pipermail/arm/2010-August/000663.html>
-       ;; for instance.
-       #f))))
+                    (let ((cpath   (search-path-as-string->list
+                                    (getenv "CPATH")))
+                          (libpath (search-path-as-string->list
+                                    (getenv "LIBRARY_PATH"))))
+                      (setenv "CPATH"
+                              (list->search-path-as-string
+                               (remove cross? cpath) ":"))
+                      (setenv "LIBRARY_PATH"
+                              (list->search-path-as-string
+                               (remove cross? libpath) ":"))
+                      #t)))
+                ,phases)
+              phases)))))))
 
 (define (cross-gcc-patches target)
   "Return GCC patches needed for TARGET."
@@ -190,14 +206,16 @@ may be either a libc package or #f.)"
   "Return a cross-compiler for TARGET, where TARGET is a GNU triplet.  Use
 XBINUTILS as the associated cross-Binutils.  If LIBC is false, then build a
 GCC that does not target a libc; otherwise, target that libc."
-  (package (inherit gcc-4.8)
+  (package (inherit %xgcc)
     (name (string-append "gcc-cross-"
                          (if libc "" "sans-libc-")
                          target))
-    (source (origin (inherit (package-source gcc-4.8))
+    (source (origin (inherit (package-source %xgcc))
               (patches
-               (cons (search-patch "gcc-cross-environment-variables.patch")
-                     (cross-gcc-patches target)))))
+               (append
+                (origin-patches (package-source %xgcc))
+                (cons (search-patch "gcc-cross-environment-variables.patch")
+                      (cross-gcc-patches target))))))
 
     ;; For simplicity, use a single output.  Otherwise libgcc_s & co. are not
     ;; found by default, etc.
@@ -214,17 +232,24 @@ GCC that does not target a libc; otherwise, target that libc."
        ,@(cross-gcc-arguments target libc)))
 
     (native-inputs
-     `(("binutils-cross" ,xbinutils)
+     `(("ld-wrapper-cross" ,(make-ld-wrapper
+                             (string-append "ld-wrapper-" target)
+                             #:target target
+                             #:binutils xbinutils))
+       ("binutils-cross" ,xbinutils)
 
        ;; Call it differently so that the builder can check whether the "libc"
        ;; input is #f.
        ("libc-native" ,@(assoc-ref %final-inputs "libc"))
 
        ;; Remaining inputs.
-       ,@(let ((inputs (append (package-inputs gcc-4.8)
+       ,@(let ((inputs (append (package-inputs %xgcc)
                                (alist-delete "libc" %final-inputs))))
            (if libc
                `(("libc" ,libc)
+                 ("xlinux-headers"                ;the target headers
+                  ,@(assoc-ref (package-propagated-inputs libc)
+                               "linux-headers"))
                  ,@inputs)
                inputs))))
 
@@ -294,12 +319,17 @@ XBINUTILS and the cross tool chain."
               #t))
           ,phases))))
 
-    ;; Shadow the native "linux-headers" because glibc's recipe expect the
+    ;; Shadow the native "linux-headers" because glibc's recipe expects the
     ;; "linux-headers" input to point to the right thing.
     (propagated-inputs `(("linux-headers" ,xlinux-headers)))
 
+    ;; FIXME: 'static-bash' should really be an input, not a native input, but
+    ;; to do that will require building an intermediate cross libc.
+    (inputs '())
+
     (native-inputs `(("cross-gcc" ,xgcc)
                      ("cross-binutils" ,xbinutils)
+                     ,@(package-inputs glibc)     ;FIXME: static-bash
                      ,@(package-native-inputs glibc)))))
 
 
