@@ -102,8 +102,8 @@
           (lambda* (#:key outputs #:allow-other-keys)
             (let ((java (string-append (assoc-ref outputs "out")
                                        "/share/java")))
-              (mkdir-p java)
-              (copy-file "swt.jar" (string-append java "/swt.jar"))) #t)
+              (install-file "swt.jar" java)
+              #t))
           (alist-delete 'configure %standard-phases))))))
     (inputs
      `(("xulrunner" ,icecat)
@@ -159,8 +159,8 @@ is implemented.")
         (alist-replace
          'build
          (lambda _
-           (setenv "JAVA_HOME"
-                   (assoc-ref %build-inputs "icedtea6"))
+           (setenv "JAVA_HOME" (string-append (assoc-ref %build-inputs "gcj")
+                                              "/lib/jvm"))
            ;; Disable tests to avoid dependency on hamcrest-core, which needs
            ;; Ant to build.  This is necessary in addition to disabling the
            ;; "check" phase, because the dependency on "test-jar" would always
@@ -174,7 +174,7 @@ is implemented.")
           'configure
           (alist-delete 'install %standard-phases))))))
     (native-inputs
-     `(("icedtea6" ,icedtea6 "jdk")))
+     `(("gcj" ,gcj)))
     (home-page "http://ant.apache.org")
     (synopsis "Build tool for Java")
     (description
@@ -187,7 +187,7 @@ build process and its dependencies, whereas Make uses Makefile format.")
 (define-public icedtea6
   (package
     (name "icedtea6")
-    (version "1.13.7")
+    (version "1.13.8")
     (source (origin
               (method url-fetch)
               (uri (string-append
@@ -195,7 +195,7 @@ build process and its dependencies, whereas Make uses Makefile format.")
                     version ".tar.xz"))
               (sha256
                (base32
-                "0fqq898h0mk554mya5z4j9p4x6sg2qj0ckqzx65x49zcjjp69jm5"))
+                "1nqaq2xjz88rfzm94l28c0cc48gdiwl8ijw9c46s14z3awwn5g7w"))
               (modules '((guix build utils)))
               (snippet
                '(substitute* "Makefile.in"
@@ -276,10 +276,7 @@ build process and its dependencies, whereas Make uses Makefile format.")
         'unpack
         (lambda* (#:key source inputs #:allow-other-keys)
           (and (zero? (system* "tar" "xvf" source))
-               (zero? (system* "tar" "xvjf"
-                               (assoc-ref inputs "ant-bootstrap")))
                (begin
-                 (patch-shebang "apache-ant-1.9.4/bin/ant")
                  (chdir (string-append ,name "-" ,version))
                  (mkdir "openjdk")
                  (with-directory-excursion "openjdk"
@@ -383,7 +380,6 @@ build process and its dependencies, whereas Make uses Makefile format.")
            (lambda* (#:key inputs #:allow-other-keys)
              (let* ((gcjdir  (assoc-ref %build-inputs "gcj"))
                     (gcjlib  (string-append gcjdir "/lib"))
-                    (antpath (string-append (getcwd) "/../apache-ant-1.9.4"))
                     ;; Get target-specific include directory so that
                     ;; libgcj-config.h is found when compiling hotspot.
                     (gcjinclude (let* ((port (open-input-pipe "gcj -print-file-name=include"))
@@ -407,9 +403,7 @@ build process and its dependencies, whereas Make uses Makefile format.")
                                       "/include"))
                (setenv "ALT_FREETYPE_LIB_PATH"
                        (string-append (assoc-ref %build-inputs "freetype")
-                                      "/lib"))
-               (setenv "PATH" (string-append antpath "/bin:"
-                                             (getenv "PATH")))))
+                                      "/lib"))))
            (alist-cons-before
             'check 'fix-test-framework
             (lambda _
@@ -528,13 +522,7 @@ build process and its dependencies, whereas Make uses Makefile format.")
                     (copy-recursively "openjdk.build/j2sdk-image" jdk)))
                 %standard-phases)))))))))))
     (native-inputs
-     `(("ant-bootstrap"
-        ,(origin
-           (method url-fetch)
-           (uri "https://www.apache.org/dist/ant/binaries/apache-ant-1.9.4-bin.tar.bz2")
-           (sha256
-            (base32
-             "1kw801p8h5x4f0g8i5yknppssrj5a3xy1aqrkpfnk22bd1snbh90"))))
+     `(("ant" ,ant)
        ("alsa-lib" ,alsa-lib)
        ("attr" ,attr)
        ("autoconf" ,autoconf)
@@ -569,16 +557,16 @@ build process and its dependencies, whereas Make uses Makefile format.")
        ("openjdk6-src"
         ,(origin
            (method url-fetch)
-           (uri "https://java.net/downloads/openjdk6/openjdk-6-src-b35-14_apr_2015.tar.gz")
+           (uri "https://java.net/downloads/openjdk6/openjdk-6-src-b36-22_jul_2015.tar.gz")
            (sha256
             (base32
-             "05glw29vy4yw9rkjy9y8wg6ybzi89gjwi19qpnfda978x02r2x5p"))))
+             "0mdckpazjijf6ggxzah2nq99lgsi0jk9pjbxhfq39b9lawvb45ln"))))
        ("lcms" ,lcms)
        ("zlib" ,zlib)
        ("gtk" ,gtk+-2)
        ("fontconfig" ,fontconfig)
        ("freetype" ,freetype)
-       ("gcj" ,gcj-4.8)))
+       ("gcj" ,gcj)))
     (home-page "http://icedtea.classpath.org")
     (synopsis "Java development kit")
     (description
@@ -588,7 +576,7 @@ build process and its dependencies, whereas Make uses Makefile format.")
     (license license:gpl2+)))
 
 (define-public icedtea7
-  (let* ((version "2.5.5")
+  (let* ((version "2.6.2")
          (drop (lambda (name hash)
                  (origin
                    (method url-fetch)
@@ -606,7 +594,7 @@ build process and its dependencies, whereas Make uses Makefile format.")
                       version ".tar.xz"))
                 (sha256
                  (base32
-                  "1irxk2ndwsfk4c1zbzb5h3rpwv2bc9bhfjvz6p4dws5476vsxrq9"))
+                  "0xi0w8gpxx3r68hyi7fb991hxb3rqfp7895nfsl4wj3sa1f5ds5y"))
                 (modules '((guix build utils)))
                 (snippet
                  '(substitute* "Makefile.in"
@@ -630,7 +618,11 @@ build process and its dependencies, whereas Make uses Makefile format.")
          #:locale "C"
          ,@(substitute-keyword-arguments (package-arguments icedtea6)
              ((#:configure-flags flags)
-              `(delete "--with-openjdk-src-dir=./openjdk" ,flags))
+              `(delete "--with-openjdk-src-dir=./openjdk"
+                       ;; TODO: package pcsc and sctp, and add to inputs
+                       (append '("--disable-system-pcsc"
+                                 "--disable-system-sctp")
+                               ,flags)))
              ((#:phases phases)
               `(modify-phases ,phases
                  (replace
@@ -734,27 +726,26 @@ build process and its dependencies, whereas Make uses Makefile format.")
                     #t))
                  (delete 'patch-patches))))))
       (native-inputs
-       `(("ant" ,ant)
-         ("openjdk-drop"
+       `(("openjdk-drop"
           ,(drop "openjdk"
-                 "03gxqn17cxwl1nspnwigacaqd28p02d45f396j5f4kkbzfnbl0ak"))
+                 "0jabxc8iw7ciz6f2qshcpla66qniy686vnxnfx3h2yw7syvas4a9"))
          ("corba-drop"
           ,(drop "corba"
-                 "0ldcckknn2f92jv1144cnn0z3wmvxln28wc00rc6xxblnjcnamzh"))
+                 "1bw22djg8mfqqn8kp8mpbj9vi4pl8dk67qwwrny67d0fvirixylj"))
          ("jaxp-drop"
           ,(drop "jaxp"
-                 "0mnjdziffcnyqlyvf8dw1hrl4kiiwmh8ia0ym417wgvnjpaihci9"))
+                 "1h3g2dwbj8ihicl73qbr4cvvc3i5bs5ckrpja1nx6g5b56xa7kcl"))
          ("jaxws-drop"
           ,(drop "jaxws"
-                 "1gkmypnhygx2mxhca3ngy620k993wi2cc1wysc0np06y1rkx1mkn"))
+                 "1m1h7455qn4pdhb5yamdl9965iz9260lzwl3njcs35vi14v7fihl"))
          ("jdk-drop"
           ,(drop "jdk"
-                 "10x43mqjfn43jlckic0nyf7apyyjyr910cdmmvy41kvw8ljhvg61"))
+                 "1wcaxf2chnlpk34q04c23im6z32dy8fr6f9giz3ih65nyvah3n3s"))
          ("langtools-drop"
           ,(drop "langtools"
-                 "0q5nqc14r6vmhxgikw3wgdcc0r9symp830v13isnv8qdjgm6kcki"))
+                 "0da3cmm8nwz7dk2sqnywvidaa0kjnyzzi33p2lkdi4415f8yhgx5"))
          ("hotspot-drop"
           ,(drop "hotspot"
-                 "1yqxfd2jwbm5y41wscyfx8h0fr3h8ny2g2mda5iwd8sikxsaj96p"))
+                 "0fn3cjhqsgbkfzychkvvw6whxil2n9dr6q0196ywxzkinny1hjcq"))
          ,@(fold alist-delete (package-native-inputs icedtea6)
-                 '("openjdk6-src" "ant-bootstrap")))))))
+                 '("openjdk6-src")))))))
