@@ -1,8 +1,9 @@
 ;;; GNU Guix --- Functional package management for GNU
-;;; Copyright © 2014 David Thompson <dthompson2@worcester.edu>
+;;; Copyright © 2014, 2015 David Thompson <dthompson2@worcester.edu>
 ;;; Copyright © 2014 Andreas Enge <andreas@enge.fr>
 ;;; Copyright © 2014 Cyrill Schenkel <cyrill.schenkel@gmail.com>
 ;;; Copyright © 2014 Ian Denhardt <ian@zenhack.net>
+;;; Copyright © 2015 Paul van der Walt <paul@denknerd.org>
 ;;;
 ;;; This file is part of GNU Guix.
 ;;;
@@ -27,6 +28,9 @@
   #:use-module (guix utils)
   #:use-module (guix build-system gnu)
   #:use-module (gnu packages avahi)
+  #:use-module (gnu packages boost)
+  #:use-module (gnu packages icu4c)
+  #:use-module (gnu packages readline)
   #:use-module (gnu packages compression)
   #:use-module (gnu packages curl)
   #:use-module (gnu packages doxygen)
@@ -35,28 +39,25 @@
   #:use-module (gnu packages mp3)
   #:use-module (gnu packages ncurses)
   #:use-module (gnu packages pkg-config)
+  #:use-module (gnu packages autotools)
   #:use-module (gnu packages pulseaudio)
   #:use-module (gnu packages databases)
   #:use-module (gnu packages video)
-  #:use-module (gnu packages xiph)
-  #:export (libmpdclient
-            mpd
-            ncmpc
-            ncmpcpp))
+  #:use-module (gnu packages xiph))
 
-(define libmpdclient
+(define-public libmpdclient
   (package
     (name "libmpdclient")
-    (version "2.9")
+    (version "2.10")
     (source (origin
               (method url-fetch)
               (uri
                (string-append "http://musicpd.org/download/libmpdclient/"
                               (car (string-split version #\.))
-                              "/libmpdclient-" version ".tar.gz"))
+                              "/libmpdclient-" version ".tar.xz"))
               (sha256
                (base32
-                "0csb9r3nlmbwpiryixjr5k33x3zqd61xjhwmlps3a6prck1n1xw2"))))
+                "10pzs9z815a8hgbbbiliapyiw82bnplsccj5irgqjw5f5plcs22g"))))
     (build-system gnu-build-system)
     (native-inputs `(("doxygen" ,doxygen)))
     (synopsis "Music Player Daemon client library")
@@ -65,27 +66,29 @@ interfacing MPD in the C, C++ & Objective C languages.")
     (home-page "http://www.musicpd.org/libs/libmpdclient/")
     (license license:bsd-3)))
 
-(define mpd
+(define-public mpd
   (package
     (name "mpd")
-    (version "0.18.8")
+    (version "0.19.10")
     (source (origin
               (method url-fetch)
               (uri
                (string-append "http://musicpd.org/download/mpd/"
                               (version-major+minor version)
-                              "/mpd-" version ".tar.gz"))
+                              "/mpd-" version ".tar.xz"))
               (sha256
                (base32
-                "1ryqh0xf76xv4mpwy1gjwy275ar4wmbzifa9ccjim9r7lk2hgp5v"))))
+                "0laqn68iggqf0h06hg282cvpd9wsjqpjfg5fnn9wk3gr48yyp1n3"))))
     (build-system gnu-build-system)
     (inputs `(("ao" ,ao)
               ("alsa-lib" ,alsa-lib)
               ("avahi" ,avahi)
+              ("boost" ,boost)
               ("curl" ,curl)
               ("ffmpeg" ,ffmpeg)
               ("flac" ,flac)
               ("glib" ,glib)
+              ("icu4c" ,icu4c)
               ("lame" ,lame)
               ("libid3tag" ,libid3tag)
               ("libmad" ,libmad)
@@ -126,19 +129,41 @@ protocol.")
     (home-page "http://www.musicpd.org/")
     (license license:gpl2)))
 
-(define ncmpc
+(define-public mpd-mpc
+  (package
+    (name "mpd-mpc")
+    (version "0.27")
+    (source (origin
+              (method url-fetch)
+              (uri
+               (string-append "http://www.musicpd.org/download/mpc/"
+                              (car (string-split version #\.))
+                              "/mpc-" version ".tar.xz"))
+              (sha256
+               (base32
+                "0r10wsqxsi07gns6mfnicvpci0sbwwj4qa9iyr1ysrgadl5bx8j5"))))
+    (build-system gnu-build-system)
+    (inputs `(("libmpdclient" ,libmpdclient)))
+    (native-inputs `(("pkg-config" ,pkg-config)))
+    (synopsis "Music Player Daemon client")
+    (description "MPC is a minimalist command line interface to MPD, the music
+player daemon.")
+    (home-page "http://www.musicpd.org/clients/mpc/")
+    (license license:gpl2)))
+
+(define-public ncmpc
   (package
     (name "ncmpc")
-    (version "0.21")
+    (version "0.24")
     (source (origin
               (method url-fetch)
               (uri
                (string-append "http://musicpd.org/download/ncmpc/"
                               (car (string-split version #\.))
-                              "/ncmpc-" version ".tar.gz"))
+                              "/ncmpc-" version ".tar.xz"))
               (sha256
                (base32
-                "1gpy6rr0awl6xgkswmr8rdvqfkrz83rmwk441c00a9d4z3zb1a16"))))
+                "1sf3nirs3mcx0r5i7acm9bsvzqzlh730m0yjg6jcyj8ln6r7cvqf"))))
     (build-system gnu-build-system)
     (inputs `(("glib" ,glib)
               ("libmpdclient" ,libmpdclient)
@@ -150,10 +175,10 @@ terminal using ncurses.")
     (home-page "http://www.musicpd.org/clients/ncmpc/")
     (license license:gpl2)))
 
-(define ncmpcpp
+(define-public ncmpcpp
   (package
     (name "ncmpcpp")
-    (version "0.5.10")
+    (version "0.6.7")
     (source (origin
               (method url-fetch)
               (uri
@@ -161,11 +186,28 @@ terminal using ncurses.")
                               version ".tar.bz2"))
               (sha256
                (base32
-                "1a54g6dary1rirrny9fd0hpxpyyffypni3mpbdpvmjnrl9v56vgz"))))
+                "0yr1ib14qkgbsv839anpzkfbwkm6gg8wv4bf98ar7q5l2p2pv008"))))
     (build-system gnu-build-system)
     (inputs `(("libmpdclient" ,libmpdclient)
-              ("ncurses" ,ncurses)))
-    (native-inputs `(("pkg-config" ,pkg-config)))
+              ("boost"  ,boost)
+              ("readline" ,readline)
+              ("ncurses" ,ncurses)
+              ("taglib" ,taglib)))
+    (native-inputs
+     `(("pkg-config" ,pkg-config)
+       ("automake" ,automake)
+       ("autoconf" ,autoconf)
+       ("libtool" ,libtool)))
+    (arguments
+     '(#:configure-flags
+       '("BOOST_LIB_SUFFIX=" "--with-taglib")
+       #:phases
+       (alist-cons-after
+        'unpack 'autogen
+        (lambda _
+          (setenv "NOCONFIGURE" "true")
+          (zero? (system* "sh" "autogen.sh")))
+        %standard-phases)))
     (synopsis "Featureful ncurses based MPD client inspired by ncmpc")
     (description "Ncmpcpp is an mpd client with a UI very similar to ncmpc,
 but it provides new useful features such as support for regular expressions
@@ -173,3 +215,27 @@ for library searches, extended song format, items filtering, the ability to
 sort playlists, and a local filesystem browser.")
     (home-page "http://ncmpcpp.rybczak.net/")
     (license license:gpl2+)))
+
+(define-public mpdscribble
+  (package
+    (name "mpdscribble")
+    (version "0.22")
+    (source (origin
+              (method url-fetch)
+              (uri (string-append "http://www.musicpd.org/download/mpdscribble/"
+                                  version "/mpdscribble-" version ".tar.gz"))
+              (sha256
+               (base32
+                "0f0ybx380x2z2g1qvdndpvcrhkrgsfqckhz3ryydq2w3pl12v27z"))))
+    (build-system gnu-build-system)
+    (inputs `(("libmpdclient" ,libmpdclient)
+              ("curl" ,curl)
+              ("glib" ,glib)))
+    (native-inputs `(("pkg-config" ,pkg-config)))
+    (synopsis "MPD client for track scrobbling")
+    (description "mpdscribble is a Music Player Daemon client which submits
+information about tracks being played to a scrobbler, such as Libre.FM.")
+    ;; musicpd.org doesn't mention mpdscribble.  It points users to this wiki
+    ;; instead.
+    (home-page "http://mpd.wikia.com/wiki/Client:Mpdscribble")
+    (license license:gpl2+)))
