@@ -22,7 +22,6 @@
   #:use-module (gnu packages compression)
   #:use-module (gnu packages gnupg)
   #:use-module (gnu packages groff)
-  #:use-module (gnu packages openssl)
   #:use-module (gnu packages elf)
   #:use-module (gnu packages guile)
   #:use-module (gnu packages pkg-config)
@@ -32,7 +31,8 @@
   #:use-module (gnu packages ncurses)
   #:autoload   (gnu packages protobuf) (protobuf)
   #:autoload   (gnu packages boost) (boost)
-  #:use-module (gnu packages which)
+  #:use-module (gnu packages base)
+  #:use-module (gnu packages tls)
   #:use-module (gnu packages)
   #:use-module (guix packages)
   #:use-module (guix download)
@@ -43,24 +43,15 @@
 (define-public libssh
   (package
     (name "libssh")
-    (version "0.6.3")
+    (version "0.6.5")
     (source (origin
               (method url-fetch)
-              (uri (string-append "https://red.libssh.org/attachments/download/87/libssh-"
-                                  version ".tar.xz"))
+              (uri (string-append
+                    "https://red.libssh.org/attachments/download/121/libssh-"
+                    version ".tar.xz"))
               (sha256
                (base32
-                "1jyaj9h1iglvn02hrvcchbx8ycjpj8b91h8mi459k7q5jp2xgd9b"))
-              (patches
-               ;; Apply the patch as discussed at
-               ;; <http://lists.gnu.org/archive/html/guix-devel/2014-10/msg00180.html>.
-               (list (origin
-                       (uri
-                        "http://git.libssh.org/projects/libssh.git/patch/?id=a033b93c616f4a81afc3fc6a017396d507d96c19")
-                       (method url-fetch)
-                       (sha256
-                        (base32
-                         "16fdkjcnhc6j7qxz9575fzr9044927ws5259yh0xb9yv1q14j0vh")))))))
+                "0b6wyx6bwbb8jpn8x4rhlrdiqwqrwrs0mxjmrnqykm9kw1ijgm8g"))))
     (build-system cmake-build-system)
     (arguments
      '(#:configure-flags '("-DWITH_GCRYPT=ON")
@@ -68,11 +59,7 @@
        ;; TODO: Add 'CMockery' and '-DWITH_TESTING=ON' for the test suite.
        #:tests? #f))
     (inputs `(("zlib" ,zlib)
-               ;; Link against an older gcrypt, because libssh tries to access
-               ;; fields of 'gcry_thread_cbs' that are now private:
-               ;; src/threads.c:72:26: error: 'struct gcry_thread_cbs' has no member named 'mutex_init'
-              ("libgcrypt", libgcrypt-1.5)))
-    (native-inputs `(("patchelf" ,patchelf)))
+              ("libgcrypt", libgcrypt)))
     (synopsis "SSH client library")
     (description
      "libssh is a C library implementing the SSHv2 and SSHv1 protocol for
@@ -123,16 +110,18 @@ a server that supports the SSH-2 protocol.")
 (define-public openssh
   (package
    (name "openssh")
-   (version "6.7p1")
+   (version "7.0p1")
    (source (origin
             (method url-fetch)
             (uri (let ((tail (string-append name "-" version ".tar.gz")))
-                   (list (string-append "ftp://ftp.fr.openbsd.org/pub/OpenBSD/OpenSSH/portable/"
+                   (list (string-append "http://openbsd.cs.fau.de/pub/OpenBSD/OpenSSH/portable/"
                                         tail)
-                         (string-append "ftp://ftp2.fr.openbsd.org/pub/OpenBSD/OpenSSH/portable/"
+                         (string-append "http://ftp.fr.openbsd.org/pub/OpenBSD/OpenSSH/portable/"
+                                        tail)
+                         (string-append "http://ftp2.fr.openbsd.org/pub/OpenBSD/OpenSSH/portable/"
                                         tail))))
             (sha256 (base32
-                     "01smf9pvn2sk5qs80gkmc9acj07ckawi1b3xxyysp3c5mr73ky5j"))))
+                     "1rc52jyc5v5b8j9kvasrnz9vnj9b0i7fw4nqac8wix0r794k4ngx"))))
    (build-system gnu-build-system)
    (inputs `(("groff" ,groff)
              ("openssl" ,openssl)
@@ -150,9 +139,10 @@ a server that supports the SSH-2 protocol.")
        (alist-cons-before
         'check 'patch-tests
         (lambda _
-          ;; remove tests that require the user sshd
+          ;; remove 't-exec' regress target which requires user 'sshd'
           (substitute* "regress/Makefile"
-            (("t10 t-exec") "t10")))
+            (("^(REGRESS_TARGETS=.*) t-exec(.*)" all pre post)
+             (string-append pre post))))
        (alist-replace
         'install
         (lambda* (#:key (make-flags '()) #:allow-other-keys)
@@ -179,22 +169,24 @@ The connection layer multiplexes many different concurrent channels over the
 authenticated connection and allows tunneling of login sessions and
 TCP-forwarding.  It provides a flow control service for these channels.
 Additionally, various channel-specific options can be negotiated.")
-   (license (license:bsd-style "file://LICENSE"
+   (license (license:non-copyleft "file://LICENSE"
                                "See LICENSE in the distribution."))
    (home-page "http://www.openssh.org/")))
 
 (define-public guile-ssh
   (package
     (name "guile-ssh")
-    (version "0.7.1")
+    (version "0.8.0")
     (source (origin
+              ;; ftp://memory-heap.org/software/guile-ssh/guile-ssh-VERSION.tar.gz
+              ;; exists, but the server appears to be too slow and unreliable.
               (method git-fetch)
               (uri (git-reference
                     (url "https://github.com/artyom-poptsov/libguile-ssh.git")
-                    (commit "e216e1d78bf93a9753ea813f930cac5e68e52180")))
+                    (commit (string-append "v" version))))
               (sha256
                (base32
-                "1sbxhmynmpwfjwb3dp6lrc3cxi5kffqmb6klhx7wnkgqxvs61lsw"))))
+                "1ld2khzylaylhqfsfcvbxs95frvm8pkr7dq40ia1wwn9c349fcdv"))))
     (build-system gnu-build-system)
     (arguments
      '(#:phases (alist-cons-after
@@ -216,7 +208,8 @@ Additionally, various channel-specific options can be negotiated.")
                       ;; Make sure it works.
                       (setenv "GUILE_LOAD_PATH" guiledir)
                       (setenv "GUILE_LOAD_COMPILED_PATH" guiledir)
-                      (system* "guile" "-c" "(use-modules (ssh session))")))
+                      (zero?
+                       (system* "guile" "-c" "(use-modules (ssh session))"))))
                   %standard-phases))
        #:configure-flags (list (string-append "--with-guilesitedir="
                                               (assoc-ref %outputs "out")
@@ -232,7 +225,7 @@ Additionally, various channel-specific options can be negotiated.")
                      ("which" ,which)))
     (inputs `(("guile" ,guile-2.0)
               ("libssh" ,libssh)
-              ("libgcrypt" ,libgcrypt-1.5)))
+              ("libgcrypt" ,libgcrypt)))
     (synopsis "Guile bindings to libssh")
     (description
      "Guile-SSH is a library that provides access to the SSH protocol for
@@ -259,7 +252,7 @@ libssh library.")
      '(#:phases
        (alist-replace
         'configure
-        (lambda* (#:key outputs inputs system target
+        (lambda* (#:key outputs inputs system build target
                         #:allow-other-keys #:rest args)
           (let* ((configure (assoc-ref %standard-phases 'configure))
                  (prefix (assoc-ref outputs "out"))
@@ -267,9 +260,8 @@ libssh library.")
                  ;; Set --build and --host flags as the provided config.guess
                  ;; is not able to detect them
                  (flags `(,(string-append "--prefix=" prefix)
-                          ,(string-append "--build=" system)
-                          ,(string-append "--host="
-                                          (or target system)))))
+                          ,(string-append "--build=" build)
+                          ,(string-append "--host=" (or target build)))))
             (setenv "CONFIG_SHELL" bash)
             (zero? (apply system* bash
                           (string-append "." "/configure")
@@ -287,14 +279,14 @@ in future and NTLM based authentication is most likey never be supported.")
 (define-public mosh
   (package
     (name "mosh")
-    (version "1.2.4")
+    (version "1.2.5")
     (source (origin
               (method url-fetch)
-              (uri (string-append "http://mosh.mit.edu/mosh-"
+              (uri (string-append "https://mosh.mit.edu/mosh-"
                                   version ".tar.gz"))
               (sha256
                (base32
-                "0inzfmqrab3n97m7rrmhd4xh3hjz0xva2sfl5m06w11668r0skg7"))))
+                "1qsb0y882yfgwnpy6f98pi5xqm6kykdsrxzvaal37hs7szjhky0s"))))
     (build-system gnu-build-system)
     (arguments
      '(#:phases (alist-cons-after
