@@ -28,24 +28,24 @@
 (define-public tmux
   (package
     (name "tmux")
-    (version "1.7")
+    (version "2.0")
     (source (origin
              (method url-fetch)
              (uri (string-append
-                    "mirror://sourceforge/tmux/tmux/tmux-"
+                    "https://github.com/tmux/tmux/releases/download/"
                     version "/tmux-" version ".tar.gz"))
              (sha256
               (base32
-               "0ywy1x2g905hmhkdz418ik42lcvnhnwr8fv63rcqczfg27d6nd38"))))
+               "0qnkda8kb747vmbldjpb23ksv9pq3s65xhh1ja5rdsmh8r24npvr"))))
     (build-system gnu-build-system)
     (inputs
      `(("libevent" ,libevent)
        ("ncurses" ,ncurses)))
-    (home-page "http://tmux.sourceforge.net/")
+    (home-page "http://tmux.github.io/")
     (synopsis "Terminal multiplexer")
     (description
      "tmux is a terminal multiplexer: it enables a number of terminals (or
 windows), each running a separate program, to be created, accessed, and
-controlled from a single screen. tmux may be detached from a screen and
+controlled from a single screen.  tmux may be detached from a screen and
 continue running in the background, then later reattached.")
     (license isc)))
