@@ -29,6 +29,10 @@ trap 'rm -r "$tmpdir"' EXIT
 
 mkdir "$tmpdir"
 
+# 'guix environment' launches /bin/sh if 'SHELL' is unset, so export 'SHELL'
+# since we know it's valid (build environments lack /bin/sh.)
+export SHELL
+
 # Check the environment variables for the bootstrap Guile.
 guix environment --ad-hoc guile-bootstrap --pure --search-paths > "$tmpdir/a"
 guix environment --ad-hoc guile-bootstrap:out --pure --search-paths > "$tmpdir/b"
@@ -40,7 +44,15 @@ test "`wc -l < "$tmpdir/a"`" = 1
 cmp "$tmpdir/a" "$tmpdir/b"
 
 # Make sure the exit value is preserved.
-if guix environment --ad-hoc guile-bootstrap --pure -E 'guile -c "(exit 42)"'
+if guix environment --ad-hoc guile-bootstrap --pure -- guile -c '(exit 42)'
+then
+    false
+else
+    test $? = 42
+fi
+
+# Same as above, but with deprecated -E flag.
+if guix environment --ad-hoc guile-bootstrap --pure -E "guile -c '(exit 42)'"
 then
     false
 else
@@ -66,7 +78,7 @@ then
     # as returned by '--search-paths'.
     guix environment -e '(@@ (gnu packages commencement) gnu-make-boot0)'	\
 	 --no-substitutes --pure						\
-         --exec='echo $PATH $CPATH $LIBRARY_PATH' > "$tmpdir/b"
+         -- /bin/sh -c 'echo $PATH $CPATH $LIBRARY_PATH' > "$tmpdir/b"
     ( . "$tmpdir/a" ; echo $PATH $CPATH $LIBRARY_PATH ) > "$tmpdir/c"
     cmp "$tmpdir/b" "$tmpdir/c"
 
@@ -89,4 +101,29 @@ then
 
     # Make sure the "debug" output is not listed.
     if grep -E "$make_boot0_debug" "$tmpdir/a"; then false; else true; fi
+
+    # Compute the build environment for the initial GNU Make, but add in the
+    # bootstrap Guile as an ad-hoc addition.
+    guix environment -e '(@@ (gnu packages commencement) gnu-make-boot0)' \
+         --ad-hoc guile-bootstrap --no-substitutes --search-paths \
+         --pure > "$tmpdir/a"
+
+    # Make sure the bootstrap binaries are all listed where they belong.
+    cat $tmpdir/a
+    grep -E '^export PATH=.*-bootstrap-binaries-0/bin'      "$tmpdir/a"
+    grep -E '^export PATH=.*-guile-bootstrap-2.0/bin'       "$tmpdir/a"
+    grep -E '^export CPATH=.*-gcc-bootstrap-0/include'      "$tmpdir/a"
+    grep -E '^export CPATH=.*-glibc-bootstrap-0/include'    "$tmpdir/a"
+    grep -E '^export LIBRARY_PATH=.*-glibc-bootstrap-0/lib' "$tmpdir/a"
+
+    # Make sure a package list can be used with -e.
+    expr_list_test_code="
+(list (@@ (gnu packages commencement) gnu-make-boot0)
+      (@ (gnu packages bootstrap) %bootstrap-guile))"
+
+    guix environment --ad-hoc --no-substitutes --search-paths --pure \
+         -e "$expr_list_test_code" > "$tmpdir/a"
+
+    grep -E '^export PATH=.*-make-boot0-4.1/bin'      "$tmpdir/a"
+    grep -E '^export PATH=.*-guile-bootstrap-2.0/bin' "$tmpdir/a"
 fi
