@@ -84,8 +84,7 @@ baz > 13.37")
          ('version "1.0.0")
          ('source ('origin
                     ('method 'url-fetch)
-                    ('uri ('string-append "https://example.com/foo-"
-                                          'version ".tar.gz"))
+                    ('uri (pypi-uri "foo" version))
                     ('sha256
                      ('base32
                       (? string? hash)))))
